# coding=utf-8
# Copyright 2019 The Google Research Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Example code that computes FVD for some empty frames.

The FVD for this setup should be around 131.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import tensorflow as tf
import frechet_video_distance as fvd

import argparse
import os
import numpy as np
from PIL import Image

# Number of videos must be divisible by 16.
NUMBER_OF_VIDEOS = 16
VIDEO_LENGTH = 15


def main(argv):
    parser = argparse.ArgumentParser()
    parser.add_argument('base_path')
    parser.add_argument('--num_images', default=100)
    parser.add_argument('--sequence_length', default=5)
    parser.add_argument('--context_length', default=2)
    args = parser.parse_args()
    total_score = 0

    for i in range(args.num_images // 16):
        gt_images = []
        pred_images = []
        for k in range(16):
            gt_images.append([])
            pred_images.append([])
            for j in range(args.sequence_length):
                gt_name = os.path.join(args.base_path, "context_images", "context_image_{}_0_{}.png".format(i*16+k, j+args.context_length))
                pred_name = os.path.join(args.base_path, "gen_images", "gen_image_{:05d}_00_{:02d}.png".format(i*16+k, j))
                gt_image = Image.open(gt_name)
                pred_image = Image.open(pred_name)
                gt_images[-1].append(np.asarray(gt_image))
                pred_images[-1].append(np.asarray(pred_image))
            while len(pred_images[-1]) < VIDEO_LENGTH:
                pred_images[-1].append(np.asarray(pred_image))
                gt_images[-1].append(np.asarray(gt_image))
        gt_images = np.asarray(gt_images)
        pred_images = np.asarray(pred_images)
#        print("gt_images", gt_images.shape)
#        print("pred_images", pred_images.shape)

#        gt_images = np.zeros([NUMBER_OF_VIDEOS, 15, 64, 48, 3])
#        pred_images = np.zeros([NUMBER_OF_VIDEOS, 15, 64, 48, 3])
#        pred_images[:, :10, :, :, :] = 255

        with tf.Graph().as_default():
            first_set_of_videos = tf.convert_to_tensor(gt_images, np.float32)
            second_set_of_videos = tf.convert_to_tensor(pred_images, np.float32)
#            first_set_of_videos = tf.zeros([NUMBER_OF_VIDEOS, VIDEO_LENGTH, 64, 64, 3])
#            second_set_of_videos = tf.ones([NUMBER_OF_VIDEOS, VIDEO_LENGTH, 64, 64, 3]
#                                          ) * 255

            result = fvd.calculate_fvd(
                fvd.create_id3_embedding(fvd.preprocess(first_set_of_videos,
                                                        (224, 224))),
                fvd.create_id3_embedding(fvd.preprocess(second_set_of_videos,
                                                        (224, 224))))

            print("running sessiong\n\n\n\n\n\n\n\n\n\n\n\n")
            with tf.Session() as sess:
                sess.run(tf.global_variables_initializer())
                sess.run(tf.tables_initializer())
                fvd_score = sess.run(result)
                print("FVD is: %.2f." % fvd_score)

    print("total score", total_score)
    print("total score:", total_score / (args.num_images // 16))

if __name__ == "__main__":
    print("Version", tf.__version__)
#    tf.app.run(main)
    main(None)
